@@ -7,7 +7,9 @@
 from enum import Enum
 from typing import List, Optional
 
-Comments = Optional[List['Comment']]
+from antlr4 import Token
+
+Comments = Optional[List["Comment"]]
 
 
 def _equal_dicts(d1, d2, ignore_keys):
@@ -22,16 +24,29 @@ def _equal_dicts(d1, d2, ignore_keys):
 
 
 class Node:
-    """Base class for AST node.
-    """
+    """Base class for AST node."""
 
-    def __init__(self, name: str, comments: Comments = None):
+    def __init__(
+        self,
+        name: str,
+        comments: Comments = None,
+        first_token: Optional[Token] = None,
+        last_token: Optional[Token] = None,
+    ):
+        """
+
+        Args:
+            name: Node display name
+            comments: Optional comments
+            first_token: First Antlr token
+            last_token: Last Antlr token
+        """
         if comments is None:
             comments = []
         self._name: str = name
-        self.comments: List[str] = comments
-        self.start_char: int = None  # start character offset
-        self.stop_char: int = None  # stop character offset
+        self.comments: Comments = comments
+        self._first_token: Optional[Token] = first_token
+        self._last_token: Optional[Token] = last_token
 
     @property
     def display_name(self) -> str:
@@ -39,43 +54,77 @@ class Node:
 
     def __eq__(self, other) -> bool:
         if isinstance(self, other.__class__):
-            return _equal_dicts(self.__dict__, other.__dict__, ["start_char", "stop_char"])
+            return _equal_dicts(
+                self.__dict__, other.__dict__, ["_first_token", "_last_token"]
+            )
         return False
 
+    @property
+    def first_token(self) -> Optional[Token]:
+        return self._first_token
+
+    @first_token.setter
+    def first_token(self, val):
+        self._first_token = val
+
+    @property
+    def last_token(self) -> Optional[Token]:
+        return self._last_token
+
+    @last_token.setter
+    def last_token(self, val):
+        self._last_token = val
+
+    @property
+    def start_char(self) -> Optional[int]:
+        return self._first_token.start if self._first_token else None
+
+    @property
+    def stop_char(self) -> Optional[int]:
+        return self._last_token.stop if self._last_token else None
+
+    @property
+    def line(self) -> Optional[int]:
+        """Line number."""
+        return self._first_token.line if self._first_token else None
+
     def to_json(self) -> any:
-        return {self._name: {k: v for k, v in self.__dict__.items() if not k.startswith('_') and v}}
+        return {
+            self._name: {
+                **{
+                    k: v
+                    for k, v in self.__dict__.items()
+                    if not k.startswith("_") and v
+                },
+                **{
+                    "start_char": self.start_char,
+                    "stop_char": self.stop_char,
+                    "line": self.line,
+                },
+            }
+        }
 
 
 class Comment(Node):
-    def __init__(self, s: str, is_multi_line: bool = False):
-        super(Comment, self).__init__('Comment')
+    def __init__(self, s: str, is_multi_line: bool = False, **kwargs):
+        super().__init__("Comment", **kwargs)
         self.s: str = s
         self.is_multi_line: bool = is_multi_line
 
-    def __eq__(self, other):
-        if isinstance(self, other.__class__):
-            return _equal_dicts(self.__dict__, other.__dict__, [])
-        return False
-
 
 class Statement(Node):
-    """Base class for Lua statement.
-    """
-    pass
+    """Base class for Lua statement."""
 
 
 class Expression(Node):
-    """Define a Lua expression.
-    """
-    pass
+    """Define a Lua expression."""
 
 
 class Block(Node):
-    """Define a Lua Block.
-    """
+    """Define a Lua Block."""
 
-    def __init__(self, body: List[Statement]):
-        super(Block, self).__init__('Block')
+    def __init__(self, body: List[Statement], **kwargs):
+        super().__init__("Block", **kwargs)
         self.body: List[Statement] = body
 
 
@@ -86,20 +135,18 @@ class Chunk(Node):
         body (`Block`): Chunk body.
     """
 
-    def __init__(self, body: Block, comments: Comments = None):
-        super(Chunk, self).__init__('Chunk', comments)
+    def __init__(self, body: Block, **kwargs):
+        super(Chunk, self).__init__("Chunk", **kwargs)
         self.body = body
 
 
-'''
+"""
 Left Hand Side expression.
-'''
+"""
 
 
 class Lhs(Expression):
-    """Define a Lua Left Hand Side expression.
-    """
-    pass
+    """Define a Lua Left Hand Side expression."""
 
 
 class Name(Lhs):
@@ -109,8 +156,8 @@ class Name(Lhs):
         id (`string`): Id.
     """
 
-    def __init__(self, identifier: str):
-        super(Name, self).__init__('Name')
+    def __init__(self, identifier: str, **kwargs):
+        super(Name, self).__init__("Name", **kwargs)
         self.id: str = identifier
 
 
@@ -127,16 +174,22 @@ class Index(Lhs):
         value (`string`): Id.
     """
 
-    def __init__(self, idx: Expression, value: Name, notation: IndexNotation = IndexNotation.DOT):
-        super(Index, self).__init__('Index')
+    def __init__(
+        self,
+        idx: Expression,
+        value: Name,
+        notation: IndexNotation = IndexNotation.DOT,
+        **kwargs
+    ):
+        super(Index, self).__init__("Index", **kwargs)
         self.idx: Name = idx
         self.value: Expression = value
         self.notation: IndexNotation = notation
 
 
-''' ----------------------------------------------------------------------- '''
-''' Statements                                                              '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" Statements                                                              """
+""" ----------------------------------------------------------------------- """
 
 
 class Assign(Statement):
@@ -148,8 +201,8 @@ class Assign(Statement):
 
     """
 
-    def __init__(self, targets: List[Node], values: List[Node], comments: Comments = None):
-        super(Assign, self).__init__('Assign', comments)
+    def __init__(self, targets: List[Node], values: List[Node], **kwargs):
+        super().__init__("Assign", **kwargs)
         self.targets: List[Node] = targets
         self.values: List[Node] = values
 
@@ -162,9 +215,9 @@ class LocalAssign(Assign):
         values (`list<Node>`): List of values.
     """
 
-    def __init__(self, targets: List[Node], values: List[Node], comments: Comments = None):
-        super(LocalAssign, self).__init__(targets, values, comments)
-        self._name: str = 'LocalAssign'
+    def __init__(self, targets: List[Node], values: List[Node], **kwargs):
+        super().__init__(targets, values, **kwargs)
+        self._name: str = "LocalAssign"
 
 
 class While(Statement):
@@ -175,8 +228,8 @@ class While(Statement):
         body (`Block`): List of statements to execute.
     """
 
-    def __init__(self, test: Expression, body: Block):
-        super(While, self).__init__('While')
+    def __init__(self, test: Expression, body: Block, **kwargs):
+        super().__init__("While", **kwargs)
         self.test: Expression = test
         self.body: Block = body
 
@@ -188,8 +241,8 @@ class Do(Statement):
         body (`Block`): List of statements to execute.
     """
 
-    def __init__(self, body: Block):
-        super(Do, self).__init__('Do')
+    def __init__(self, body: Block, **kwargs):
+        super().__init__("Do", **kwargs)
         self.body: Block = body
 
 
@@ -201,8 +254,8 @@ class Repeat(Statement):
         body (`Block`): List of statements to execute.
     """
 
-    def __init__(self, body: Block, test: Expression, comments: Comments = None):
-        super(Repeat, self).__init__('Repeat', comments)
+    def __init__(self, body: Block, test: Expression, **kwargs):
+        super().__init__("Repeat", **kwargs)
         self.body: Block = body
         self.test: Expression = test
 
@@ -216,8 +269,8 @@ class ElseIf(Statement):
         orelse (`list<Statement> or ElseIf`): List of statements or ElseIf if test if false.
     """
 
-    def __init__(self, test: Node, body: Block, orelse):
-        super(ElseIf, self).__init__('ElseIf')
+    def __init__(self, test: Node, body: Block, orelse, **kwargs):
+        super().__init__("ElseIf", **kwargs)
         self.test: Node = test
         self.body: Block = body
         self.orelse = orelse
@@ -232,9 +285,10 @@ class If(Statement):
         orelse (`list<Statement> or ElseIf`): List of statements or ElseIf if test if false.
     """
 
-    def __init__(self, test: Expression, body: Block, orelse: List[Statement] or ElseIf,
-                 comments: Comments = None):
-        super(If, self).__init__('If', comments)
+    def __init__(
+        self, test: Expression, body: Block, orelse: List[Statement] or ElseIf, **kwargs
+    ):
+        super().__init__("If", **kwargs)
         self.test: Expression = test
         self.body: Block = body
         self.orelse = orelse
@@ -247,8 +301,8 @@ class Label(Statement):
         id (`Name`): Label name.
     """
 
-    def __init__(self, label_id: Name):
-        super(Label, self).__init__('Label')
+    def __init__(self, label_id: Name, **kwargs):
+        super(Label, self).__init__("Label", **kwargs)
         self.id: Name = label_id
 
 
@@ -259,26 +313,23 @@ class Goto(Statement):
         label (`Name`): Label node.
     """
 
-    def __init__(self, label: Name, comments: Comments = None):
-        super(Goto, self).__init__('Goto', comments)
+    def __init__(self, label: Name, **kwargs):
+        super(Goto, self).__init__("Goto", **kwargs)
         self.label: Name = label
 
 
 class SemiColon(Statement):
-    """Define the semi-colon lua statement.
-    """
+    """Define the semi-colon lua statement."""
 
-    def __init__(self):
-        super(SemiColon, self).__init__('SemiColon')
+    def __init__(self, **kwargs):
+        super(SemiColon, self).__init__("SemiColon", **kwargs)
 
 
 class Break(Statement):
-    """Define the break lua statement.
+    """Define the break lua statement."""
 
-    """
-
-    def __init__(self):
-        super(Break, self).__init__('Break')
+    def __init__(self, **kwargs):
+        super(Break, self).__init__("Break", **kwargs)
 
 
 class Return(Statement):
@@ -288,8 +339,8 @@ class Return(Statement):
         values (`list<Expression>`): Values to return.
     """
 
-    def __init__(self, values):
-        super(Return, self).__init__('Return')
+    def __init__(self, values, **kwargs):
+        super(Return, self).__init__("Return", **kwargs)
         self.values = values
 
 
@@ -304,9 +355,16 @@ class Fornum(Statement):
         body (`Block`): List of statements to execute.
     """
 
-    def __init__(self, target: Name, start: Expression, stop: Expression, step: Expression, body: Block,
-                 comments: Comments = None):
-        super(Fornum, self).__init__('Fornum', comments)
+    def __init__(
+        self,
+        target: Name,
+        start: Expression,
+        stop: Expression,
+        step: Expression,
+        body: Block,
+        **kwargs
+    ):
+        super(Fornum, self).__init__("Fornum", **kwargs)
         self.target: Name = target
         self.start: Expression = start
         self.stop: Expression = stop
@@ -323,8 +381,10 @@ class Forin(Statement):
         targets (`list<Name>`): Start index value.
     """
 
-    def __init__(self, body: Block, iter: List[Expression], targets: List[Name], comments: Comments = None):
-        super(Forin, self).__init__('Forin', comments)
+    def __init__(
+        self, body: Block, iter: List[Expression], targets: List[Name], **kwargs
+    ):
+        super(Forin, self).__init__("Forin", **kwargs)
         self.body: Block = body
         self.iter: List[Expression] = iter
         self.targets: List[Name] = targets
@@ -338,8 +398,8 @@ class Call(Statement):
         args (`list<Expression>`): Function call arguments.
     """
 
-    def __init__(self, func: Expression, args: List[Expression], comments: Comments = None):
-        super(Call, self).__init__('Call', comments)
+    def __init__(self, func: Expression, args: List[Expression], **kwargs):
+        super(Call, self).__init__("Call", **kwargs)
         self.func: Expression = func
         self.args: List[Expression] = args
 
@@ -353,8 +413,10 @@ class Invoke(Statement):
         args (`list<Expression>`): Function call arguments.
     """
 
-    def __init__(self, source: Expression, func: Expression, args: List[Expression]):
-        super(Invoke, self).__init__('Invoke')
+    def __init__(
+        self, source: Expression, func: Expression, args: List[Expression], **kwargs
+    ):
+        super(Invoke, self).__init__("Invoke", **kwargs)
         self.source: Expression = source
         self.func: Expression = func
         self.args: List[Expression] = args
@@ -369,8 +431,8 @@ class Function(Statement):
         body (`Block`): List of statements to execute.
     """
 
-    def __init__(self, name: Expression, args: List[Expression], body: Block):
-        super(Function, self).__init__('Function')
+    def __init__(self, name: Expression, args: List[Expression], body: Block, **kwargs):
+        super(Function, self).__init__("Function", **kwargs)
         self.name: Expression = name
         self.args: List[Expression] = args
         self.body: Block = body
@@ -385,8 +447,8 @@ class LocalFunction(Statement):
         body (`list<Statement>`): List of statements to execute.
     """
 
-    def __init__(self, name: Expression, args: List[Expression], body: Block):
-        super(LocalFunction, self).__init__('LocalFunction')
+    def __init__(self, name: Expression, args: List[Expression], body: Block, **kwargs):
+        super(LocalFunction, self).__init__("LocalFunction", **kwargs)
         self.name: Expression = name
         self.args: List[Expression] = args
         self.body: Block = body
@@ -402,46 +464,49 @@ class Method(Statement):
         body (`Block`): List of statements to execute.
     """
 
-    def __init__(self, source: Expression, name: Expression, args: List[Expression], body: Block,
-                 comments: Comments = None):
-        super(Method, self).__init__('Method', comments)
+    def __init__(
+        self,
+        source: Expression,
+        name: Expression,
+        args: List[Expression],
+        body: Block,
+        **kwargs
+    ):
+        super(Method, self).__init__("Method", **kwargs)
         self.source: Expression = source
         self.name: Expression = name
         self.args: List[Expression] = args
         self.body: Block = body
 
 
-''' ----------------------------------------------------------------------- '''
-''' Lua Expression                                                          '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" Lua Expression                                                          """
+""" ----------------------------------------------------------------------- """
 
-''' ----------------------------------------------------------------------- '''
-''' Types and values                                                        '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" Types and values                                                        """
+""" ----------------------------------------------------------------------- """
 
 
 class Nil(Expression):
-    """Define the Lua nil expression.
-    """
+    """Define the Lua nil expression."""
 
-    def __init__(self):
-        super(Nil, self).__init__('Nil')
+    def __init__(self, **kwargs):
+        super(Nil, self).__init__("Nil", **kwargs)
 
 
 class TrueExpr(Expression):
-    """Define the Lua true expression.
-    """
+    """Define the Lua true expression."""
 
-    def __init__(self):
-        super(TrueExpr, self).__init__('True')
+    def __init__(self, **kwargs):
+        super(TrueExpr, self).__init__("True", **kwargs)
 
 
 class FalseExpr(Expression):
-    """Define the Lua false expression.
-    """
+    """Define the Lua false expression."""
 
-    def __init__(self):
-        super(FalseExpr, self).__init__('False')
+    def __init__(self, **kwargs):
+        super(FalseExpr, self).__init__("False", **kwargs)
 
 
 NumberType = int or float
@@ -454,18 +519,16 @@ class Number(Expression):
         n (`int|float`): Numeric value.
     """
 
-    def __init__(self, n: NumberType):
-        super(Number, self).__init__('Number')
+    def __init__(self, n: NumberType, **kwargs):
+        super(Number, self).__init__("Number", **kwargs)
         self.n: NumberType = n
 
 
 class Varargs(Expression):
-    """Define the Lua Varargs expression (...).
+    """Define the Lua Varargs expression (...)."""
 
-    """
-
-    def __init__(self):
-        super(Varargs, self).__init__('Varargs')
+    def __init__(self, **kwargs):
+        super(Varargs, self).__init__("Varargs", **kwargs)
 
 
 class StringDelimiter(Enum):
@@ -482,8 +545,13 @@ class String(Expression):
         delimiter (`StringDelimiter`): The string delimiter
     """
 
-    def __init__(self, s: str, delimiter: StringDelimiter = StringDelimiter.SINGLE_QUOTE):
-        super(String, self).__init__('String')
+    def __init__(
+        self,
+        s: str,
+        delimiter: StringDelimiter = StringDelimiter.SINGLE_QUOTE,
+        **kwargs
+    ):
+        super(String, self).__init__("String", **kwargs)
         self.s: str = s
         self.delimiter: StringDelimiter = delimiter
 
@@ -496,9 +564,14 @@ class Field(Expression):
         value (`Expression`): Value.
     """
 
-    def __init__(self, key: Expression, value: Expression, comments: Comments = None,
-                 between_brackets: bool = False):
-        super(Field, self).__init__('Field', comments)
+    def __init__(
+        self,
+        key: Expression,
+        value: Expression,
+        between_brackets: bool = False,
+        **kwargs
+    ):
+        super().__init__("Field", **kwargs)
         self.key: Expression = key
         self.value: Expression = value
         self.between_brackets: bool = between_brackets
@@ -511,17 +584,16 @@ class Table(Expression):
         fields (`list<Field>`): Table fields.
     """
 
-    def __init__(self, fields: List[Field]):
-        super(Table, self).__init__('Table')
+    def __init__(self, fields: List[Field], **kwargs):
+        super().__init__("Table", **kwargs)
         self.fields: List[Field] = fields
 
 
 class Dots(Expression):
-    """Define the Lua dots (...) expression.
-    """
+    """Define the Lua dots (...) expression."""
 
-    def __init__(self):
-        super(Dots, self).__init__('Dots')
+    def __init__(self, **kwargs):
+        super().__init__("Dots", **kwargs)
 
 
 class AnonymousFunction(Expression):
@@ -532,21 +604,19 @@ class AnonymousFunction(Expression):
         body (`Block`): List of statements to execute.
     """
 
-    def __init__(self, args: List[Expression], body: Block):
-        super(AnonymousFunction, self).__init__('AnonymousFunction')
+    def __init__(self, args: List[Expression], body: Block, **kwargs):
+        super(AnonymousFunction, self).__init__("AnonymousFunction", **kwargs)
         self.args: List[Expression] = args
         self.body: Block = body
 
 
-''' ----------------------------------------------------------------------- '''
-''' Operators                                                               '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" Operators                                                               """
+""" ----------------------------------------------------------------------- """
 
 
 class Op(Expression):
-    """Base class for Lua operators.
-    """
-    pass
+    """Base class for Lua operators."""
 
 
 class BinaryOp(Op):
@@ -557,20 +627,19 @@ class BinaryOp(Op):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, name, left: Expression, right: Expression):
-        super(BinaryOp, self).__init__(name)
+    def __init__(self, name, left: Expression, right: Expression, **kwargs):
+        super(BinaryOp, self).__init__(name, **kwargs)
         self.left: Expression = left
         self.right: Expression = right
 
 
-''' ----------------------------------------------------------------------- '''
-''' 3.4.1 – Arithmetic Operators                                            '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" 3.4.1 – Arithmetic Operators                                            """
+""" ----------------------------------------------------------------------- """
 
 
 class AriOp(BinaryOp):
     """Base class for Arithmetic Operators"""
-    pass
 
 
 class AddOp(AriOp):
@@ -581,8 +650,8 @@ class AddOp(AriOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(AddOp, self).__init__('AddOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("AddOp", left, right, **kwargs)
 
 
 class SubOp(AriOp):
@@ -593,8 +662,8 @@ class SubOp(AriOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(SubOp, self).__init__('SubOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("SubOp", left, right, **kwargs)
 
 
 class MultOp(AriOp):
@@ -605,8 +674,8 @@ class MultOp(AriOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(MultOp, self).__init__('MultOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("MultOp", left, right, **kwargs)
 
 
 class FloatDivOp(AriOp):
@@ -617,8 +686,8 @@ class FloatDivOp(AriOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(FloatDivOp, self).__init__('FloatDivOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("FloatDivOp", left, right, **kwargs)
 
 
 class FloorDivOp(AriOp):
@@ -629,8 +698,8 @@ class FloorDivOp(AriOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(FloorDivOp, self).__init__('FloorDivOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("FloorDivOp", left, right, **kwargs)
 
 
 class ModOp(AriOp):
@@ -641,8 +710,8 @@ class ModOp(AriOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(ModOp, self).__init__('ModOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("ModOp", left, right, **kwargs)
 
 
 class ExpoOp(AriOp):
@@ -653,19 +722,17 @@ class ExpoOp(AriOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(ExpoOp, self).__init__('ExpoOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("ExpoOp", left, right, **kwargs)
 
 
-''' ----------------------------------------------------------------------- '''
-''' 3.4.2 – Bitwise Operators                                               '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" 3.4.2 – Bitwise Operators                                               """
+""" ----------------------------------------------------------------------- """
 
 
 class BitOp(BinaryOp):
-    """Base class for bitwise Operators.
-    """
-    pass
+    """Base class for bitwise Operators."""
 
 
 class BAndOp(BitOp):
@@ -676,8 +743,8 @@ class BAndOp(BitOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(BAndOp, self).__init__('BAndOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("BAndOp", left, right, **kwargs)
 
 
 class BOrOp(BitOp):
@@ -688,8 +755,8 @@ class BOrOp(BitOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(BOrOp, self).__init__('BOrOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("BOrOp", left, right, **kwargs)
 
 
 class BXorOp(BitOp):
@@ -700,8 +767,8 @@ class BXorOp(BitOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(BXorOp, self).__init__('BXorOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("BXorOp", left, right, **kwargs)
 
 
 class BShiftROp(BitOp):
@@ -712,8 +779,8 @@ class BShiftROp(BitOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(BShiftROp, self).__init__('BShiftROp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("BShiftROp", left, right, **kwargs)
 
 
 class BShiftLOp(BitOp):
@@ -724,19 +791,17 @@ class BShiftLOp(BitOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(BShiftLOp, self).__init__('BShiftLOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("BShiftLOp", left, right, **kwargs)
 
 
-''' ----------------------------------------------------------------------- '''
-''' 3.4.4 – Relational Operators                                            '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" 3.4.4 – Relational Operators                                            """
+""" ----------------------------------------------------------------------- """
 
 
 class RelOp(BinaryOp):
-    """Base class for Lua relational operators.
-    """
-    pass
+    """Base class for Lua relational operators."""
 
 
 class LessThanOp(RelOp):
@@ -747,8 +812,8 @@ class LessThanOp(RelOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(LessThanOp, self).__init__('RLtOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("RLtOp", left, right, **kwargs)
 
 
 class GreaterThanOp(RelOp):
@@ -759,8 +824,8 @@ class GreaterThanOp(RelOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(GreaterThanOp, self).__init__('RGtOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("RGtOp", left, right, **kwargs)
 
 
 class LessOrEqThanOp(RelOp):
@@ -771,8 +836,8 @@ class LessOrEqThanOp(RelOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(LessOrEqThanOp, self).__init__('RLtEqOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("RLtEqOp", left, right, **kwargs)
 
 
 class GreaterOrEqThanOp(RelOp):
@@ -783,8 +848,8 @@ class GreaterOrEqThanOp(RelOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(GreaterOrEqThanOp, self).__init__('RGtEqOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("RGtEqOp", left, right, **kwargs)
 
 
 class EqToOp(RelOp):
@@ -795,8 +860,8 @@ class EqToOp(RelOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(EqToOp, self).__init__('REqOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("REqOp", left, right, **kwargs)
 
 
 class NotEqToOp(RelOp):
@@ -807,19 +872,17 @@ class NotEqToOp(RelOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(NotEqToOp, self).__init__('RNotEqOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("RNotEqOp", left, right, **kwargs)
 
 
-''' ----------------------------------------------------------------------- '''
-''' 3.4.5 – Logical Operators                                               '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" 3.4.5 – Logical Operators                                               """
+""" ----------------------------------------------------------------------- """
 
 
 class LoOp(BinaryOp):
-    """Base class for logical operators.
-    """
-    pass
+    """Base class for logical operators."""
 
 
 class AndLoOp(LoOp):
@@ -830,8 +893,8 @@ class AndLoOp(LoOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(AndLoOp, self).__init__('LAndOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("LAndOp", left, right, **kwargs)
 
 
 class OrLoOp(LoOp):
@@ -842,13 +905,13 @@ class OrLoOp(LoOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(OrLoOp, self).__init__('LOrOp', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("LOrOp", left, right, **kwargs)
 
 
-''' ----------------------------------------------------------------------- '''
-''' 3.4.6 Concat operators                                                  '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" 3.4.6 Concat operators                                                  """
+""" ----------------------------------------------------------------------- """
 
 
 class Concat(BinaryOp):
@@ -859,13 +922,13 @@ class Concat(BinaryOp):
         right (`Expression`): Right expression.
     """
 
-    def __init__(self, left: Expression, right: Expression):
-        super(Concat, self).__init__('Concat', left, right)
+    def __init__(self, left: Expression, right: Expression, **kwargs):
+        super().__init__("Concat", left, right, **kwargs)
 
 
-''' ----------------------------------------------------------------------- '''
-''' Unary operators                                                         '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" Unary operators                                                         """
+""" ----------------------------------------------------------------------- """
 
 
 class UnaryOp(Expression):
@@ -875,8 +938,8 @@ class UnaryOp(Expression):
         operand (`Expression`): Operand.
     """
 
-    def __init__(self, name: str, operand: Expression):
-        super(UnaryOp, self).__init__(name)
+    def __init__(self, name: str, operand: Expression, **kwargs):
+        super().__init__(name, **kwargs)
         self.operand = operand
 
 
@@ -887,8 +950,8 @@ class UMinusOp(UnaryOp):
         operand (`Expression`): Operand.
     """
 
-    def __init__(self, operand: Expression):
-        super(UMinusOp, self).__init__('UMinusOp', operand)
+    def __init__(self, operand: Expression, **kwargs):
+        super().__init__("UMinusOp", operand, **kwargs)
 
 
 class UBNotOp(UnaryOp):
@@ -898,8 +961,8 @@ class UBNotOp(UnaryOp):
         operand (`Expression`): Operand.
     """
 
-    def __init__(self, operand: Expression):
-        super(UBNotOp, self).__init__('UBNotOp', operand)
+    def __init__(self, operand: Expression, **kwargs):
+        super().__init__("UBNotOp", operand, **kwargs)
 
 
 class ULNotOp(UnaryOp):
@@ -909,18 +972,17 @@ class ULNotOp(UnaryOp):
         operand (`Expression`): Operand.
     """
 
-    def __init__(self, operand: Expression):
-        super(ULNotOp, self).__init__('ULNotOp', operand)
+    def __init__(self, operand: Expression, **kwargs):
+        super().__init__("ULNotOp", operand, **kwargs)
 
 
-''' ----------------------------------------------------------------------- '''
-''' 3.4.7 – The Length Operator                                             '''
-''' ----------------------------------------------------------------------- '''
+""" ----------------------------------------------------------------------- """
+""" 3.4.7 – The Length Operator                                             """
+""" ----------------------------------------------------------------------- """
 
 
 class ULengthOP(UnaryOp):
-    """Length operator.
-    """
+    """Length operator."""
 
-    def __init__(self, operand: Expression):
-        super(ULengthOP, self).__init__('ULengthOp', operand)
+    def __init__(self, operand: Expression, **kwargs):
+        super().__init__("ULengthOp", operand, **kwargs)
