@@ -1,19 +1,22 @@
-from antlr4 import InputStream, CommonTokenStream
-from luaparser.parser.LuaLexer import LuaLexer
-from luaparser.astnodes import *
-from enum import Enum
 import ast
 import re
-from typing import List
+
+from antlr4 import InputStream, CommonTokenStream
+
+from luaparser.astnodes import *
+from luaparser.parser.LuaLexer import LuaLexer
+from typing import List, Tuple
 from antlr4.Token import Token
 
 
 class SyntaxException(Exception):
     def __init__(self, user_msg, token=None):
         if token:
-            message = '(' + str(token.line) + ',' + str(token.start) + '): Error: ' + user_msg
+            message = (
+                "(" + str(token.line) + "," + str(token.start) + "): Error: " + user_msg
+            )
         else:
-            message = 'Error: ' + user_msg
+            message = "Error: " + user_msg
         super().__init__(message)
 
 
@@ -97,16 +100,73 @@ class Tokens:
     LongBracket = 64
 
 
-LITERAL_NAMES = ["<INVALID>",
-                 "'and'", "'break'", "'do'", "'else'", "'elseif'", "'end'", "'false'",
-                 "'for'", "'function'", "'goto'", "'if'", "'in'", "'local'",
-                 "'nil'", "'not'", "'or'", "'repeat'", "'return'", "'then'",
-                 "'true'", "'until'", "'while'", "'+'", "'-'", "'*'", "'/'",
-                 "'//'", "'%'", "'^'", "'#'", "'=='", "'~='", "'<='", "'>='",
-                 "'<'", "'>'", "'='", "'&'", "'|'", "'~'", "'>>'", "'<<'", "'('",
-                 "')'", "'{'", "'}'", "'['", "']'", "'::'", "':'", "','", "'...'",
-                 "'..'", "'.'", "';'", "NAME", "NUMBER", "STRING", "COMMENT", "LINE_COMMENT",
-                 "SPACE", "NEWLINE", "SHEBANG", "LONG_BRACKET"]
+LITERAL_NAMES = [
+    "<INVALID>",
+    "'and'",
+    "'break'",
+    "'do'",
+    "'else'",
+    "'elseif'",
+    "'end'",
+    "'false'",
+    "'for'",
+    "'function'",
+    "'goto'",
+    "'if'",
+    "'in'",
+    "'local'",
+    "'nil'",
+    "'not'",
+    "'or'",
+    "'repeat'",
+    "'return'",
+    "'then'",
+    "'true'",
+    "'until'",
+    "'while'",
+    "'+'",
+    "'-'",
+    "'*'",
+    "'/'",
+    "'//'",
+    "'%'",
+    "'^'",
+    "'#'",
+    "'=='",
+    "'~='",
+    "'<='",
+    "'>='",
+    "'<'",
+    "'>'",
+    "'='",
+    "'&'",
+    "'|'",
+    "'~'",
+    "'>>'",
+    "'<<'",
+    "'('",
+    "')'",
+    "'{'",
+    "'}'",
+    "'['",
+    "']'",
+    "'::'",
+    "':'",
+    "','",
+    "'...'",
+    "'..'",
+    "'.'",
+    "';'",
+    "NAME",
+    "NUMBER",
+    "STRING",
+    "COMMENT",
+    "LINE_COMMENT",
+    "SPACE",
+    "NEWLINE",
+    "SHEBANG",
+    "LONG_BRACKET",
+]
 
 
 def _listify(obj):
@@ -117,10 +177,7 @@ def _listify(obj):
 
 
 class Builder:
-    CLOSING_TOKEN = [
-        Tokens.END,
-        Tokens.CBRACE,
-        Tokens.CPAR]
+    CLOSING_TOKEN = [Tokens.END, Tokens.CBRACE, Tokens.CPAR]
 
     HIDDEN_TOKEN = [
         Tokens.SHEBANG,
@@ -128,7 +185,8 @@ class Builder:
         Tokens.COMMENT,
         Tokens.NEWLINE,
         Tokens.SPACE,
-        -2]
+        -2,
+    ]
 
     REL_OPERATORS = [
         Tokens.LT,
@@ -136,19 +194,20 @@ class Builder:
         Tokens.LTEQ,
         Tokens.GTEQ,
         Tokens.NEQ,
-        Tokens.EQ]
+        Tokens.EQ,
+    ]
 
     def __init__(self, source):
         self._stream = CommonTokenStream(LuaLexer(InputStream(source)))
         # contains a list of CommonTokens
         self._line_count: int = 0
         self._right_index: int = 0
-        self._last_expr_type: int = None
+        self._last_expr_type: Optional[int] = None
 
         # following stack are used to backup values
         self._index_stack: List[int] = []
         self._right_index_stack: List[int] = []
-        self.text: str = ''  # last token text
+        self.text: str = ""  # last token text
         self.type: int = -1  # last token type
 
         # contains expected token in case of invalid input code
@@ -160,6 +219,11 @@ class Builder:
         self._hidden_handled: bool = False
         self._hidden_handled_stack: List[bool] = []
 
+    @property
+    def _LT(self) -> Token:
+        """Last token that was consumed in next_i*_* method."""
+        return self._stream.LT(-1)
+
     def process(self) -> Chunk:
         node = self.parse_chunk()
 
@@ -203,7 +267,9 @@ class Builder:
         self._comments_index_stack.append(len(self.comments))
         self._hidden_handled_stack.append(self._hidden_handled)
 
-    def next_is_rc(self, type_to_seek: int, hidden_right: bool = True) -> Optional[Token]:
+    def next_is_rc(
+        self, type_to_seek: int, hidden_right: bool = True
+    ) -> Optional[Token]:
         token = self._stream.LT(1)
         tok_type: int = token.type
         self._right_index = self._stream.index
@@ -271,12 +337,25 @@ class Builder:
             for t in tokens:
                 if not self._hidden_handled:
                     if t.type == Tokens.LINE_COMMENT:
-                        self.comments.append(Comment(t.text))
+                        self.comments.append(
+                            Comment(
+                                t.text,
+                                first_token=t,
+                                last_token=t,
+                            )
+                        )
                     elif t.type == Tokens.COMMENT:
-                        self.comments.append(Comment(t.text, True))
+                        self.comments.append(
+                            Comment(
+                                t.text,
+                                True,
+                                first_token=t,
+                                last_token=t,
+                            )
+                        )
                     elif t.type == Tokens.NEWLINE:
                         # append n time a None value (indicate newline)
-                        self.comments += t.text.count('\n') * [None]
+                        self.comments += t.text.count("\n") * [None]
 
         self._hidden_handled = True
 
@@ -286,12 +365,25 @@ class Builder:
             for t in tokens:
                 if not self._hidden_handled:
                     if t.type == Tokens.LINE_COMMENT:
-                        self.comments.append(Comment(t.text))
+                        self.comments.append(
+                            Comment(
+                                t.text,
+                                first_token=t,
+                                last_token=t,
+                            )
+                        )
                     elif t.type == Tokens.COMMENT:
-                        self.comments.append(Comment(t.text, True))
+                        self.comments.append(
+                            Comment(
+                                t.text,
+                                True,
+                                first_token=t,
+                                last_token=t,
+                            )
+                        )
                     elif t.type == Tokens.NEWLINE:
                         # append n time a None value (indicate newline)
-                        self.comments += t.text.count('\n') * [None]
+                        self.comments += t.text.count("\n") * [None]
 
         self._hidden_handled = True
 
@@ -301,8 +393,7 @@ class Builder:
         return comments
 
     def get_comments_followed_by_blank_line(self) -> Comments:
-        """ Returns comments followed by a blank line.
-        """
+        """Returns comments followed by a blank line."""
         if not self.comments:
             return []
 
@@ -311,16 +402,16 @@ class Builder:
 
         # search first comment
         while idx < len(self.comments) and self.comments[idx] is None:
-            idx = idx + 1
+            idx += 1
         # get comments starting from this index
         while idx < len(self.comments) and self.comments[idx] is not None:
             comments.append(self.comments[idx])
-            idx = idx + 1
+            idx += 1
         # check if followed by a blank line
         if idx + 1 < len(self.comments):
             if self.comments[idx] is None and self.comments[idx + 1] is None:
                 # clean list
-                self.comments = self.comments[idx + 2:]
+                self.comments = self.comments[idx + 2 :]
                 return comments
         return []
 
@@ -344,11 +435,16 @@ class Builder:
             types_str.append(LITERAL_NAMES[type_to_seek])
 
         raise SyntaxException(
-            "Expecting one of " + ', '.join(types_str) + ' at line ' + str(token.line) + ', column ' + str(
-                token.column))
+            "Expecting one of "
+            + ", ".join(types_str)
+            + " at line "
+            + str(token.line)
+            + ", column "
+            + str(token.column)
+        )
 
     def parse_chunk(self) -> Chunk or None:
-        self._stream.LT(1)
+        first_token: Token = self._stream.LT(1)
         self.handle_hidden_left()
         comments = self.get_comments_followed_by_blank_line()
         block = self.parse_block()
@@ -356,10 +452,16 @@ class Builder:
             token = self._stream.LT(1)
             if token.type == -1:
                 # do not consume EOF
-                return Chunk(block, comments)
+                return Chunk(
+                    block,
+                    comments=comments,
+                    first_token=first_token,
+                    last_token=self._LT,
+                )
         return False
 
     def parse_block(self) -> Block:
+        t: Token = self._stream.LT(1)
         statements = []
 
         while True:
@@ -372,22 +474,27 @@ class Builder:
         stat = self.parse_ret_stat()
         if stat:
             statements.append(stat)
-        return Block(statements)
+        return Block(
+            statements,
+            first_token=t,
+            last_token=statements[-1].last_token if statements else None,
+        )
 
     def parse_stat(self) -> Statement or None:
         comments = self.get_comments()
 
-        stat = \
-            self.parse_assignment() or \
-            self.parse_var() or \
-            self.parse_while_stat() or \
-            self.parse_repeat_stat() or \
-            self.parse_local() or \
-            self.parse_goto_stat() or \
-            self.parse_if_stat() or \
-            self.parse_for_stat() or \
-            self.parse_function() or \
-            self.parse_label()
+        stat = (
+            self.parse_assignment()
+            or self.parse_var()
+            or self.parse_while_stat()
+            or self.parse_repeat_stat()
+            or self.parse_local()
+            or self.parse_goto_stat()
+            or self.parse_if_stat()
+            or self.parse_for_stat()
+            or self.parse_function()
+            or self.parse_label()
+        )
 
         if stat:
             stat.comments = comments
@@ -410,24 +517,31 @@ class Builder:
     def parse_ret_stat(self) -> Return or bool:
         self.save()
         if self.next_is_rc(Tokens.RETURN):
+            t: Token = self._LT
             expr_list = self.parse_expr_list()  # optional
             # consume optional token
             if self.next_is(Tokens.SEMCOL):
                 self.next_is_rc(Tokens.SEMCOL)
 
             self.success()
-            return Return(expr_list)
+            return Return(expr_list, first_token=t, last_token=self._LT)
         return self.failure()
 
     def parse_assignment(self) -> Assign or bool:
         self.save()
+        t: Token = self._stream.LT(1)
         targets = self.parse_var_list()
         if targets:
             if self.next_is_rc(Tokens.ASSIGN):
                 values = self.parse_expr_list()
                 if values:
                     self.success()
-                    return Assign(targets, values)
+                    return Assign(
+                        targets,
+                        values,
+                        first_token=t,
+                        last_token=self._LT,
+                    )
                 else:
                     self.abort()
 
@@ -461,6 +575,8 @@ class Builder:
         if root:
             tail = self.parse_tail()
             while tail:
+                tail.first_token = root.first_token
+
                 if isinstance(tail, Call):
                     tail.func = root
                 elif isinstance(tail, Index):
@@ -468,7 +584,13 @@ class Builder:
                 elif isinstance(tail, Invoke):
                     tail.source = root
                 else:
-                    tail = Call(root, _listify(tail))
+                    args = _listify(tail)
+                    tail = Call(
+                        root,
+                        args,
+                        first_token=root.first_token,
+                        last_token=args[-1].last_token if args else None,
+                    )
                 root = tail
 
                 tail = self.parse_tail()
@@ -485,18 +607,36 @@ class Builder:
         self.save()
         if self.next_is_rc(Tokens.DOT) and self.next_is_rc(Tokens.NAME, False):
             self.success()
-            return Index(Name(self.text), Name(""))  # value must be set in parent
+            return Index(
+                Name(
+                    self.text,
+                    first_token=self._LT,
+                    last_token=self._LT,
+                ),
+                # value must be set in parent
+                Name(""),
+                last_token=self._LT,
+            )
 
         self.failure_save()
         if self.next_is_rc(Tokens.OBRACK):
             expr = self.parse_expr()
             if expr and self.next_is_rc(Tokens.CBRACK, False):
                 self.success()
-                return Index(expr, Name(""), notation=IndexNotation.SQUARE)  # value must be set in parent
+                return Index(
+                    expr,
+                    Name(""),
+                    notation=IndexNotation.SQUARE,
+                    # value must be set in parent
+                )
 
         self.failure_save()
         if self.next_is_rc(Tokens.COL) and self.next_is_rc(Tokens.NAME):
-            name = Name(self.text)
+            name = Name(
+                self.text,
+                first_token=self._LT,
+                last_token=self._LT,
+            )
             if self.next_is_rc(Tokens.OPAR):
                 expr_list = self.parse_expr_list() or []
                 if self.next_is_rc(Tokens.CPAR, False):
@@ -506,7 +646,11 @@ class Builder:
 
         self.failure_save()
         if self.next_is_rc(Tokens.COL) and self.next_is_rc(Tokens.NAME):
-            name = Name(self.text)
+            name = Name(
+                self.text,
+                first_token=self._LT,
+                last_token=self._LT,
+            )
             table = self.parse_table_constructor(False)
             if table:
                 self.success()
@@ -515,9 +659,15 @@ class Builder:
 
         self.failure_save()
         if self.next_is_rc(Tokens.COL) and self.next_is_rc(Tokens.NAME):
-            name = Name(self.text)
+            name = Name(
+                self.text,
+                first_token=self._LT,
+                last_token=self._LT,
+            )
             if self.next_is_rc(Tokens.STRING, False):
                 string = self.parse_lua_str(self.text)
+                string.first_token = self._LT.start
+                string.last_token = self._LT
                 self.success()
                 # noinspection PyTypeChecker
                 return Invoke(None, name, [string])
@@ -535,7 +685,9 @@ class Builder:
             if tokens:
                 for t in tokens:
                     if t.type == Tokens.NEWLINE and not self.prev_is(Tokens.SEMCOL):
-                        raise SyntaxException('Ambiguous syntax detected', self._stream.LT(-1))
+                        raise SyntaxException(
+                            "Ambiguous syntax detected", self._stream.LT(-1)
+                        )
 
         if self.next_is_rc(Tokens.OPAR, False):
             self.handle_hidden_right()
@@ -543,7 +695,7 @@ class Builder:
             if self.next_is_rc(Tokens.CPAR, False):
                 self.success()
                 # noinspection PyTypeChecker
-                return Call(None, expr_list)
+                return Call(None, expr_list, last_token=self._LT)
 
         self.failure_save()
         table = self.parse_table_constructor(False)
@@ -554,6 +706,8 @@ class Builder:
         self.failure_save()
         if self.next_is_rc(Tokens.STRING, False):
             string = self.parse_lua_str(self.text)
+            string.first_token = self._LT
+            string.last_token = self._LT
             self.success()
             return string
 
@@ -646,20 +800,29 @@ class Builder:
                     self.failure()
 
                 self.success()
-                return LocalAssign(targets, values)
+                return LocalAssign(
+                    targets,
+                    values,
+                    first_token=start_token,
+                    last_token=values[-1].last_token if values else None,
+                )
 
             self.save()
 
             if self.next_is_rc(Tokens.FUNCTION) and self.next_is_rc(Tokens.NAME):
-                name = Name(self.text)
+                name = Name(
+                    self.text,
+                    first_token=self._LT,
+                    last_token=self._LT,
+                )
                 body = self.parse_func_body()
                 if body:
                     self.success()
                     self.success()
                     node = LocalFunction(name, body[0], body[1])
                     self.handle_hidden_right()
-                    node.start_char = start_token.start
-                    node.stop_char = body[1].stop_char
+                    node.first_token = start_token
+                    node.last_token = body[1].last_token
                     return node
             self.failure()
             self.abort()
@@ -670,7 +833,13 @@ class Builder:
         self.save()
         if self.next_is_rc(Tokens.GOTO) and self.next_is_rc(Tokens.NAME):
             self.success()
-            return Goto(Name(self.text))
+            return Goto(
+                Name(
+                    self.text,
+                    first_token=self._LT,
+                    last_token=self._LT,
+                )
+            )
         return self.failure()
 
     def parse_if_stat(self) -> If or bool:
@@ -731,7 +900,11 @@ class Builder:
         if self.next_is_rc(Tokens.FOR):
             self.save()
             if self.next_is_rc(Tokens.NAME):
-                target = Name(self.text)
+                target = Name(
+                    self.text,
+                    first_token=self._LT,
+                    last_token=self._LT,
+                )
                 if self.next_is_rc(Tokens.ASSIGN):
                     start = self.parse_expr()
                     if start and self.next_is_rc(Tokens.COMMA):
@@ -739,7 +912,9 @@ class Builder:
                         if stop:
                             step = 1
                             # optional step
-                            if self.next_is(Tokens.COMMA) and self.next_is_rc(Tokens.COMMA):
+                            if self.next_is(Tokens.COMMA) and self.next_is_rc(
+                                Tokens.COMMA
+                            ):
                                 step = self.parse_expr()
 
                             body = self.parse_do_block()
@@ -773,15 +948,24 @@ class Builder:
             if names:
                 self.save()
                 if self.next_is_rc(Tokens.COL) and self.next_is_rc(Tokens.NAME):
-                    name = Name(self.text)
+                    name = Name(
+                        self.text,
+                        first_token=self._LT,
+                        last_token=self._LT,
+                    )
                     func_body = self.parse_func_body()
                     if func_body:
                         self.success()
                         self.success()
-                        node = Method(names, name, func_body[0], func_body[1])
+                        node = Method(
+                            names,
+                            name,
+                            func_body[0],
+                            func_body[1],
+                            first_token=start_token,
+                            last_token=func_body[1].last_token,
+                        )
                         self.handle_hidden_right()
-                        node.start_char = start_token.start
-                        node.stop_char = func_body[1].stop_char
                         return node
 
                 self.failure()
@@ -789,10 +973,14 @@ class Builder:
                 func_body = self.parse_func_body()
                 if func_body:
                     self.success()
-                    node = Function(names, func_body[0], func_body[1])
+                    node = Function(
+                        names,
+                        func_body[0],
+                        func_body[1],
+                        first_token=start_token,
+                        last_token=func_body[1].last_token,
+                    )
                     self.handle_hidden_right()
-                    node.start_char = start_token.start
-                    node.stop_char = func_body[1].stop_char
                     return node
             self.abort()
 
@@ -801,12 +989,23 @@ class Builder:
     def parse_names(self) -> Name or Index or bool:
         self.save()
         if self.next_is_rc(Tokens.NAME):
-            root = Name(self.text)
+            root = Name(
+                self.text,
+                first_token=self._LT,
+                last_token=self._LT,
+            )
             while True:
                 self.save()
                 if self.next_is_rc(Tokens.DOT) and self.next_is_rc(Tokens.NAME):
                     self.success()
-                    child = Index(Name(self.text), root)
+                    child = Index(
+                        Name(
+                            self.text,
+                            first_token=self._LT,
+                            last_token=self._LT,
+                        ),
+                        root,
+                    )
                     root = child
                 else:
                     self.failure()
@@ -830,7 +1029,7 @@ class Builder:
                         self._expected = []
                         token = self.next_is_rc(Tokens.END, False)
                         if token:
-                            body.stop_char = token.stop
+                            body.last_token = token
                             self.success()
                             return args, body
                         else:
@@ -843,8 +1042,7 @@ class Builder:
         param_list: List[Expression] = self.parse_name_list()
         if param_list:
             self.save()
-            if self.next_is_rc(Tokens.COMMA) and \
-                    self.next_is_rc(Tokens.VARARGS):
+            if self.next_is_rc(Tokens.COMMA) and self.next_is_rc(Tokens.VARARGS):
                 self.success()
                 param_list.append(Varargs())
                 return param_list
@@ -864,11 +1062,23 @@ class Builder:
         self.save()
         names: List[Name] = []
         if self.next_is_rc(Tokens.NAME):
-            names.append(Name(self.text))
+            names.append(
+                Name(
+                    self.text,
+                    first_token=self._LT,
+                    last_token=self._LT,
+                )
+            )
             while True:
                 self.save()
                 if self.next_is_rc(Tokens.COMMA) and self.next_is_rc(Tokens.NAME):
-                    names.append(Name(self.text))
+                    names.append(
+                        Name(
+                            self.text,
+                            first_token=self._LT,
+                            last_token=self._LT,
+                        )
+                    )
                     self.success()
                 else:
                     self.failure()
@@ -880,7 +1090,11 @@ class Builder:
     def parse_label(self) -> Label or bool:
         self.save()
         if self.next_is_rc(Tokens.COLCOL) and self.next_is_rc(Tokens.NAME):
-            name = Name(self.text)
+            name = Name(
+                self.text,
+                first_token=self._LT,
+                last_token=self._LT,
+            )
             if self.next_is_rc(Tokens.COLCOL):
                 self.success()
                 return Label(name)
@@ -900,7 +1114,11 @@ class Builder:
         self.save()
         if self.next_is_rc(Tokens.NAME):
             self.success()
-            return Name(self.text)
+            return Name(
+                self.text,
+                first_token=self._LT,
+                last_token=self._LT,
+            )
         return self.failure()
 
     def parse_expr(self) -> Expression or bool:
@@ -1037,10 +1255,7 @@ class Builder:
         if left:
             while True:
                 self.save()
-                if self.next_in_rc([Tokens.MULT,
-                                    Tokens.DIV,
-                                    Tokens.MOD,
-                                    Tokens.FLOOR]):
+                if self.next_in_rc([Tokens.MULT, Tokens.DIV, Tokens.MOD, Tokens.FLOOR]):
                     op = self.type
                     right = self.parse_bitwise_expr()
                     if right:
@@ -1070,11 +1285,15 @@ class Builder:
         if left:
             while True:
                 self.save()
-                if self.next_in_rc([Tokens.BITAND,
-                                    Tokens.BITOR,
-                                    Tokens.BITNOT,
-                                    Tokens.BITRSHIFT,
-                                    Tokens.BITRLEFT]):
+                if self.next_in_rc(
+                    [
+                        Tokens.BITAND,
+                        Tokens.BITOR,
+                        Tokens.BITNOT,
+                        Tokens.BITRSHIFT,
+                        Tokens.BITRLEFT,
+                    ]
+                ):
                     op = self.type
                     right = self.parse_unary_expr()
                     if right:
@@ -1103,31 +1322,35 @@ class Builder:
     def parse_unary_expr(self) -> Expression or bool:
         self.save()
         if self.next_is_rc(Tokens.MINUS):
+            t: Token = self._LT
             expr = self.parse_unary_expr()
             if expr:
                 self.success()
-                return UMinusOp(expr)
+                return UMinusOp(expr, first_token=t, last_token=t)
 
         self.failure_save()
         if self.next_is_rc(Tokens.LENGTH):
+            t: Token = self._LT
             expr = self.parse_pow_expr()
             if expr:
                 self.success()
-                return ULengthOP(expr)
+                return ULengthOP(expr, first_token=t, last_token=t)
 
         self.failure_save()
         if self.next_is_rc(Tokens.NOT):
+            t: Token = self._LT
             expr = self.parse_unary_expr()
             if expr:
                 self.success()
-                return ULNotOp(expr)
+                return ULNotOp(expr, first_token=t, last_token=t)
 
         self.failure_save()
         if self.next_is_rc(Tokens.BITNOT):
+            t: Token = self._LT
             expr = self.parse_unary_expr()
             if expr:
                 self.success()
-                return UBNotOp(expr)
+                return UBNotOp(expr, first_token=t, last_token=t)
 
         self.failure_save()
         expr = self.parse_pow_expr()
@@ -1180,25 +1403,32 @@ class Builder:
             except:
                 # exception occurs with leading zero number: 002
                 number = float(self.text)
-            return Number(number)
+            return Number(
+                number,
+                first_token=self._LT,
+                last_token=self._LT,
+            )
 
         if self.next_is(Tokens.STRING) and self.next_is_rc(Tokens.STRING):
-            return self.parse_lua_str(self.text)
+            string = self.parse_lua_str(self.text)
+            string.first_token = self._LT
+            string.last_token = self._LT
+            return string
 
         if self.next_is(Tokens.NIL) and self.next_is_rc(Tokens.NIL):
-            return Nil()
+            return Nil(first_token=self._LT, last_token=self._LT)
 
         if self.next_is(Tokens.TRUE) and self.next_is_rc(Tokens.TRUE):
-            return TrueExpr()
+            return TrueExpr(first_token=self._LT, last_token=self._LT)
 
         if self.next_is(Tokens.FALSE) and self.next_is_rc(Tokens.FALSE):
-            return FalseExpr()
+            return FalseExpr(first_token=self._LT, last_token=self._LT)
         return None
 
     @staticmethod
     def parse_lua_str(lua_str) -> String:
         delimiter: StringDelimiter = StringDelimiter.SINGLE_QUOTE
-        p = re.compile(r'^\[=+\[(.*)\]=+\]')  # nested quote pattern
+        p = re.compile(r"^\[=+\[(.*)]=+]")  # nested quote pattern
         # try remove double quote:
         if lua_str.startswith('"') and lua_str.endswith('"'):
             lua_str = lua_str[1:-1]
@@ -1219,10 +1449,16 @@ class Builder:
     def parse_function_literal(self) -> AnonymousFunction or bool:
         self.save()
         if self.next_is_rc(Tokens.FUNCTION):
+            t: Token = self._LT
             func_body = self.parse_func_body()
             if func_body:
                 self.success()
-                node = AnonymousFunction(func_body[0], func_body[1])
+                node = AnonymousFunction(
+                    func_body[0],
+                    func_body[1],
+                    first_token=t,
+                    last_token=self._LT,
+                )
                 self.handle_hidden_right()
                 return node
 
@@ -1251,7 +1487,7 @@ class Builder:
     def parse_field_list(self) -> List[Field] or bool:
         field_list = []
         self.save()
-        field = self.parse_field()
+        field, _ = self.parse_field()
         if field:
             field_list.append(field)
             while True:
@@ -1260,16 +1496,17 @@ class Builder:
                     inline_com = self.get_inline_comment()
                     if inline_com:
                         field.comments.append(inline_com)
-                    field = self.parse_field()
+                    prev_field = field
+                    field, remaining_comments = self.parse_field()
                     if field:
                         field_list.append(field)
                         self.success()
                     else:
+                        prev_field.comments.extend(remaining_comments)
                         self.success()
                         self.success()
                         return field_list
                 else:
-
                     field.comments.extend(self.get_comments())
                     self.failure()
                     break
@@ -1278,7 +1515,7 @@ class Builder:
             return field_list
         return self.failure()
 
-    def parse_field(self) -> Field or bool:
+    def parse_field(self) -> Tuple[Field or bool, Comments]:
         self.save()
 
         if self.next_is_rc(Tokens.OBRACK):
@@ -1289,17 +1526,24 @@ class Builder:
                     value = self.parse_expr()
                     if value:
                         self.success()
-                        return Field(key, value, comments, between_brackets=True)
+                        return (
+                            Field(key, value, comments=comments, between_brackets=True),
+                            comments,
+                        )
 
         self.failure_save()
         if self.next_is_rc(Tokens.NAME):
-            key = Name(self.text)
+            key = Name(
+                self.text,
+                first_token=self._LT,
+                last_token=self._LT,
+            )
             if self.next_is_rc(Tokens.ASSIGN):
                 comments = self.get_comments()
                 value = self.parse_expr()
                 if value:
                     self.success()
-                    return Field(key, value, comments)
+                    return Field(key, value, comments=comments), comments
 
         self.failure_save()
         comments = self.get_comments()
@@ -1307,9 +1551,12 @@ class Builder:
         if value:
             self.success()
             # noinspection PyTypeChecker
-            return Field(None, value, comments)  # Key will be set in parse_table_constructor
+            return (
+                Field(None, value, comments=comments),
+                [],
+            )  # Key will be set in parse_table_constructor
 
-        return self.failure()
+        return self.failure(), comments
 
     def parse_field_sep(self) -> bool:
         self.save()
